@@ -55,7 +55,6 @@ namespace Beagle.Filters {
 		{
 			// 1: Store entry names as text content
 			SetVersion (1);
-
 		}
 
 		protected override void RegisterSupportedTypes ()
@@ -121,7 +120,17 @@ namespace Beagle.Filters {
 			ArchiveEntry a_entry;
 			int count = 0;
 
-			while ((a_entry = this.get_next_entry ()) != null && count < MAX_CHILDREN) {
+			while (count < MAX_CHILDREN && (a_entry = this.get_next_entry ()) != null) {
+				++count;
+
+				// Store file names in the archive
+				AppendText (Path.GetFileName (a_entry.Name));
+				AppendWhiteSpace ();
+
+				// If this is an invalid entry (corrupt archive), skip it.
+				if (a_entry.TempFile == null)
+					continue;
+
 				// FIXME: For nested archives, create uid:foo#bar
 				// instead of uid:foo#xxx#bar (avoid duplicates ?)
 				Indexable child = new Indexable (new Uri (Uri.ToString () + "#" + a_entry.Name, true));
@@ -142,12 +151,6 @@ namespace Beagle.Filters {
 					child.AddProperty (prop);
 
 				AddChildIndexable (child);
-
-				// Store file names in the archive
-				AppendText (Path.GetFileName (a_entry.Name));
-				AppendWhiteSpace ();
-
-				++count;
 			}
 		}
 
@@ -167,22 +170,43 @@ namespace Beagle.Filters {
 
 			string filename = Path.GetTempFileName ();
 			FileStream file_stream = File.OpenWrite (filename);
+
+			//Log.Debug ("Storing archive contents in {0}", filename);
 			
 			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 384); // 384 is 0600
 			
 			BufferedStream buffered_stream = new BufferedStream (file_stream);
 
 			byte [] buffer = new byte [8192];
+			long prev_pos = -1;
 			int read;
+			int broken_count = 0;
+			bool broken_file = false;
 
 			do {
 				read = stream.Read (buffer, 0, buffer.Length);
 				if (read > 0)
 					buffered_stream.Write (buffer, 0, read);
+
+				// Lame workaround for some gzip files which loop
+				// forever with SharpZipLib.  We have to check for
+				// 
+				if (stream is GZipInputStream && read == buffer.Length) {
+					if (stream.Position == prev_pos) {
+						broken_file = true;
+						break;
+					} else
+						prev_pos = stream.Position;
+				}
 			} while (read > 0);
 
 			buffered_stream.Close ();
 
+			if (broken_file) {
+				File.Delete (filename);
+				return null;
+			}
+
 			File.SetLastWriteTimeUtc (filename, mtime);
 
 			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 256); // 256 is 0400
