@@ -76,11 +76,8 @@ namespace Beagle.Filters {
 			pc.Arguments = new string [] { "rpm", "-qp", "--queryformat", "[%{*:xml}\n]", FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
 
-			// Runs inside the child process after fork() but before exec()
-			pc.ChildProcessSetup += delegate {
-				// Let rpm run for 90 seconds, max.
-				SystemPriorities.SetResourceLimit (SystemPriorities.Resource.Cpu, 90);
-			};
+			// Let rpm run for 90 seconds, max.
+			pc.CpuLimit = 90;
 			
 			try {
 				pc.Start ();
