@@ -36,6 +36,7 @@ using GLib;
 
 using Beagle.Util;
 using Log = Beagle.Util.Log;
+using Stopwatch = Beagle.Util.Stopwatch;
 
 namespace Beagle.Daemon {
 	class BeagleDaemon {
@@ -49,6 +50,7 @@ namespace Beagle.Daemon {
 		private static bool arg_disable_scheduler = false;
 		private static bool arg_indexing_test_mode = false;
 		private static bool arg_heap_shot = false;
+		private static bool arg_heap_shot_snapshots = true;
 
 		public static bool StartServer ()
 		{
@@ -66,7 +68,7 @@ namespace Beagle.Daemon {
 
 		public static void ReplaceExisting () 
 		{
-			Logger.Log.Info ("Attempting to replace another beagled.");
+			Log.Always ("Attempting to replace another beagled.");
 
 			do {
 				ShutdownRequest request = new ShutdownRequest ();
@@ -115,7 +117,7 @@ namespace Beagle.Daemon {
 
 				int vm_rss = SystemInformation.VmRss;
 
-				if (arg_heap_shot)
+				if (arg_heap_shot && arg_heap_shot_snapshots)
 					MaybeSendSigprof (vm_rss, GC.GetTotalMemory (false));
 
 				if (vm_rss > 300 * 1024) {
@@ -315,6 +317,10 @@ namespace Beagle.Daemon {
 					arg_debug_memory = true;
 					break;
 
+				case "--no-shapshots":
+					arg_heap_shot_snapshots = false;
+					break;
+
 				case "--heap-buddy":
 				case "--debug-memory":
 					arg_debug = true;
@@ -442,12 +448,11 @@ namespace Beagle.Daemon {
 					LogLevel.Debug,
 					arg_fg);
 
-			Logger.Log.Info ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
-
-			Logger.Log.Info ("Running on {0}", SystemInformation.MonoRuntimeVersion);
-			
-			Logger.Log.Debug ("Command Line: {0}",
-					  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
+			Log.Always ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
+			Log.Always ("Running on {0}", SystemInformation.MonoRuntimeVersion);
+			Log.Always ("Using sqlite version {0}", ExternalStringsHack.SqliteVersion);
+			Log.Always ("Command Line: {0}",
+					   Environment.CommandLine != null ? Environment.CommandLine : "(null)");
 
 			if (! ExtendedAttribute.Supported) {
 				Logger.Log.Warn ("Extended attributes are not supported on this filesystem.  " +
@@ -500,7 +505,7 @@ namespace Beagle.Daemon {
 				} catch (IOException) { }
 			}
 
-			Log.Info ("Beagle daemon process shut down cleanly.");
+			Log.Always ("Beagle daemon process shut down cleanly.");
 		}
 
 		/////////////////////////////////////////////////////////////////////////////
