@@ -40,6 +40,8 @@ using LNS = Lucene.Net.Search;
 
 using Beagle;
 using Beagle.Util;
+
+using Stopwatch = Beagle.Util.Stopwatch;
 using FSQ = Beagle.Daemon.FileSystemQueryable.FileSystemQueryable;
 
 namespace Beagle.Daemon 
@@ -274,8 +276,10 @@ namespace Beagle.Daemon
 				}
 			}
 
-			// Set the IO priority so we don't slow down the system
-			IoPriority.ReduceIoPriority ();
+			// Set system priorities so we don't slow down the system
+			SystemPriorities.ReduceIoPriority ();
+			SystemPriorities.SetSchedulerPolicyBatch ();
+			SystemPriorities.Renice (19);
 			
 			driver = new LuceneIndexingDriver (arg_output, false);
 			driver.TextCache = (arg_cache_text) ? new TextCache (arg_output) : null;
