@@ -37,30 +37,42 @@ namespace Beagle.Daemon {
 
 	public class Filter {
 
+		static private bool Debug = false;
+
+		// Lucene fields allow a maximum of 10000 words
+		// Some of the words will be stop words... so a failsafe maximum of 40000 words
+		// Dont accept more words than that
+		const int MAXWORDS = 40000; // Lucene.Net.Index.IndexWriter.DEFAULT_MAX_FIELD_LENGTH * 4
+
 		// Derived classes always must have a constructor that
 		// takes no arguments.
 		public Filter () { }
 
 		//////////////////////////
 
-		private string identifier;
-
-		public string Identifier {
-			get { return identifier; }
-			set { identifier = value; }
-		}
-
-		//////////////////////////
-
-		private ArrayList supported_flavors = new ArrayList ();
+		private ArrayList supported_flavors = null;
 		
 		protected void AddSupportedFlavor (FilterFlavor flavor) 
 		{
+			// Add flavor only when called from RegisterSupportedTypes
+			if (supported_flavors == null)
+				throw new Exception ("AddSupportedFlavor() should be only called from RegisterSupportedTypes()");
+
 			supported_flavors.Add (flavor);
 		}
 
 		public ICollection SupportedFlavors {
-			get { return supported_flavors; }
+			get {
+				if (supported_flavors == null) {
+					supported_flavors = new ArrayList ();
+					RegisterSupportedTypes ();
+				}
+				return supported_flavors;
+			}
+		}
+
+		protected virtual void RegisterSupportedTypes ()
+		{
 		}
 		
 		//////////////////////////
@@ -102,6 +114,7 @@ namespace Beagle.Daemon {
 		private string this_mime_type = null;
 		private string this_extension = null;
 		private ArrayList indexable_properties = null;
+		private DateTime timestamp = DateTime.MinValue;
 
 		public string MimeType {
 			get { return this_mime_type; }
@@ -113,12 +126,19 @@ namespace Beagle.Daemon {
 			set { this_extension = value; }
 		}
 
-		// allow the filter to access the properties
+		// Allow the filter to access the properties
 		// set by indexable
 		public ArrayList IndexableProperties {
 			get { return indexable_properties; }
 			set { indexable_properties = value; }
 		}
+
+		// Allow the filter to access the timestamp,
+		// sometime filters know better
+		public DateTime Timestamp {
+			get { return timestamp; }
+			set { timestamp = value; }
+		}
 		
 		//////////////////////////
 		
@@ -135,6 +155,19 @@ namespace Beagle.Daemon {
 
 		//////////////////////////
 
+		// Filters which deal with big files, and that don't need
+		// to read in whole files may want to set this to false
+		// to avoid wasting cycles in disk wait.
+
+		private bool preload = true;
+
+		protected bool PreLoad {
+			get { return preload; }
+			set { preload = value; }
+		}
+
+		//////////////////////////
+
 		int hotCount = 0;
 		int freezeCount = 0;
 
@@ -195,28 +228,58 @@ namespace Beagle.Daemon {
 		private ArrayList textPool;
 		private ArrayList hotPool;
 		private ArrayList propertyPool;
+
+		private int word_count = 0;
+		private int hotword_count = 0;
+
+		protected bool AllowMoreWords ()
+		{
+			return (word_count < MAXWORDS);
+		}
 		
 		private bool last_was_structural_break = true;
+		const string WHITESPACE = " ";
 
-		// This two-arg AppendText() will give flexibility to
-		// filters to segregate hot-contents and
-		// normal-contents of a para and call this method with
-		// respective contents.  
-		//
-		// str : Holds both the normal-contents and hot contents.
-		// strHot: Holds only hot-contents.
-		//
-		// Ex:- suppose the actual-content is "one <b>two</b> three"
-		// str = "one two three"
-		// strHot = "two"
-		//
-		// NOTE: HotUp() or HotDown() has NO-EFFECT on this variant 
-		// of AppendText ()
-		
-		public void AppendText (string str, string strHot)
+		/* Append text to the textpool. If IsHot is true, then also add to the hottext pool.
+		 * Handles null str.
+		 */
+		public int AppendText (string str)
 		{
-			if (!IsFrozen && str != null && str != "") {
-				int i = 0;
+			if (Debug)
+				Logger.Log.Debug ("AppendText (\"{0}\")", str);
+
+			return AppendText (str, IsHot ? str : null);
+		}
+
+		/*
+		 * This two-arg AppendText() will give flexibility to
+		 * filters to segregate hot-contents and
+		 * normal-contents of a para and call this method with
+		 * respective contents.  
+		 * 
+		 * str : Holds both the normal-contents and hot contents.
+		 * strHot: Holds only hot-contents.
+		 * Both arguments can be null.
+		 * 
+		 * Ex:- suppose the actual-content is "one <b>two</b> three"
+		 * str = "one two three"
+		 * strHot = "two"
+		 * 
+		 * NOTE: HotUp() or HotDown() has NO-EFFECT on this variant 
+		 * of AppendText ()
+		 */
+
+		public int AppendText (string str, string strHot)
+		{
+			int num_words = 0;
+
+			if (Debug)
+				Logger.Log.Debug ("AppendText (\"{0}, {1}\")", str, strHot);
+
+			if (IsFrozen)
+				return 0;
+
+			if (word_count < MAXWORDS && ! string.IsNullOrEmpty (str)) {
 				string[] lines;
 
 				// Avoid unnecessary allocation of a string
@@ -225,38 +288,122 @@ namespace Beagle.Daemon {
 					lines = str.Split ('\n'); 
 					foreach (string line in lines) {
 						if (line.Length > 0) {
-							ReallyAppendText (line, null);
+							ReallyAppendText (line);
 							AppendStructuralBreak ();
 						}
 					}
 				} else 
-					ReallyAppendText (str, null);
+					ReallyAppendText (str);
+
+				num_words = StringFu.CountWords (str, 3, -1);
+				word_count += num_words;
 			}
-			ReallyAppendText (null, strHot);
+
+			if (hotword_count < MAXWORDS && ! string.IsNullOrEmpty (strHot)) {
+				ReallyAppendHotText (strHot);
+				hotword_count += StringFu.CountWords (strHot, 3, -1);
+			}
+
+			return num_words;
 		}
 		
-		public void AppendText (string str)
+		// strHot may not be null or empty - checked before
+		private void ReallyAppendHotText (string strHot)
 		{
-			//Logger.Log.Debug ("AppendText (\"{0}\")", str);
-			if (! IsFrozen && str != null && str != "")
-				AppendText (str, IsHot ? str : null);
+			hotPool.Add (strHot.Trim());
+			hotPool.Add (WHITESPACE);
 		}
 
-		// Does adding text to to text/hot pools respectively.
-		private void ReallyAppendText (string str, string strHot)
+		// str may not be null or empty - checked before
+		private void ReallyAppendText (string str)
 		{
-			if (!IsFrozen && strHot != null && strHot != "")
-				hotPool.Add (strHot.Trim()+" ");
+			textPool.Add (str);
 
-			if (str != null) {
-				textPool.Add (str);
+			if (snippetWriter != null)
+				snippetWriter.Write (str);
 
+			last_was_structural_break = false;
+		}
+
+		// Add a word followed by a whitespace. word may not be whitespace or newline.
+		public int AppendWord (string word)
+		{
+			if (Debug)
+				Logger.Log.Debug ("AppendWord (\"{0}\")", word);
+
+			return AppendWords (word, false);
+		}
+
+		// Add a line followed by a newline.
+		public int AppendLine (string line)
+		{
+			if (Debug)
+				Logger.Log.Debug ("AppendLine (\"{0}\")", line);
+
+			return AppendWords (line, true);
+		}
+
+		private int AppendWords (string words, bool is_line)
+		{
+			if (IsFrozen || string.IsNullOrEmpty (words))
+				return 0;
+
+			if (IsHot) {
+				hotPool.Add (words);
+				hotPool.Add (WHITESPACE);
+				hotword_count += StringFu.CountWords (words, 3, -1);
+			}
+
+			textPool.Add (words);
+
+			if (snippetWriter != null)
+				snippetWriter.Write (words);
+
+			if (is_line)
+				AppendStructuralBreak ();
+			else
+				AppendWhiteSpace ();
+
+			int num_words = StringFu.CountWords (words, 3, -1);
+			word_count += num_words;
+			return num_words;
+		}
+
+		/*
+		 * Adds whitespace to the textpool.
+		 */
+		public void AppendWhiteSpace ()
+		{
+			if (last_was_structural_break)
+				return;
+
+			if (Debug)
+				Logger.Log.Debug ("AppendWhiteSpace ()");
+
+			if (NeedsWhiteSpace (textPool)) {
+				textPool.Add (WHITESPACE);
 				if (snippetWriter != null)
-					snippetWriter.Write (str);
-
+					snippetWriter.Write (WHITESPACE);
 				last_was_structural_break = false;
 			}
 		}
+
+		/*
+		 * Creates a new paragraph. Mainly useful for storing cached contents.
+		 */
+		public void AppendStructuralBreak ()
+		{
+			if (snippetWriter != null && ! last_was_structural_break) {
+				snippetWriter.WriteLine ();
+				last_was_structural_break = true;
+			}
+
+			// When adding a "newline" to the textCache, we need to 
+			// append a "Whitespace" to the text pool.
+			if (NeedsWhiteSpace (textPool))
+				textPool.Add (WHITESPACE);
+		}
+
 		private bool NeedsWhiteSpace (ArrayList array)
 		{
 			if (array.Count == 0)
@@ -270,38 +417,16 @@ namespace Beagle.Daemon {
 			return true;
 		}
 
-		public void AppendWhiteSpace ()
-		{
-			if (last_was_structural_break)
-				return;
-
-			//Logger.Log.Debug ("AppendWhiteSpace ()");
-			if (NeedsWhiteSpace (textPool)) {
-				textPool.Add (" ");
-				if (snippetWriter != null)
-					snippetWriter.Write (" ");
-				last_was_structural_break = false;
-			}
-		}
-
+		/*
+		 * Adds property prop.
+		 * prop can be null or can have null value; in both cases nothing is added.
+		 */
 		public void AddProperty (Property prop)
 		{
-			if (prop != null && prop.Value != null)
+			if (prop != null && ! string.IsNullOrEmpty (prop.Value))
 				propertyPool.Add (prop);
 		}
 
-		public void AppendStructuralBreak ()
-		{
-			if (snippetWriter != null && ! last_was_structural_break) {
-				snippetWriter.WriteLine ();
-				last_was_structural_break = true;
-			}
-			// When adding a "newline" to the textCache, we need to 
-			// append a "Whitespace" to the text pool.
-			if (NeedsWhiteSpace (textPool))
-				textPool.Add (" ");
-		}
-
 		//////////////////////////
 
 		private bool isFinished = false;
@@ -374,11 +499,12 @@ namespace Beagle.Daemon {
 			tempFile = Path.GetTempFileName ();
                         FileStream file_stream = File.OpenWrite (tempFile);
 
-			//Logger.Log.Debug ("Storing text in tempFile {0}", tempFile);
+			if (Debug)
+				Logger.Log.Debug ("Storing text in tempFile {0}", tempFile);
 
                         // When we dump the contents of a reader into a file, we
                         // expect to use it again soon.
-                        FileAdvise.PreLoad (file_stream);
+			FileAdvise.PreLoad (file_stream);
 
                         // Make sure the temporary file is only readable by the owner.
                         // FIXME: There is probably a race here.  Could some malicious program
@@ -408,7 +534,8 @@ namespace Beagle.Daemon {
 			tempFile = Path.GetTempFileName ();
                         FileStream file_stream = File.OpenWrite (tempFile);
 			
-			//Logger.Log.Debug ("Storing stream in tempFile {0}", tempFile);
+			if (Debug)
+				Logger.Log.Debug ("Storing stream in tempFile {0}", tempFile);
 
                         // When we dump the contents of a reader into a file, we
                         // expect to use it again soon.
@@ -452,13 +579,15 @@ namespace Beagle.Daemon {
 								FileAccess.Read,
 								FileShare.Read);
 				
-				// Our default assumption is sequential reads.
-				// FIXME: Is this the right thing to do here?
-				FileAdvise.IncreaseReadAhead (currentStream);
+				if (preload) {
+					// Our default assumption is sequential reads.
+					// FIXME: Is this the right thing to do here?
+					FileAdvise.IncreaseReadAhead (currentStream);
 				
-				// Give the OS a hint that we will be reading this
-				// file soon.
-				FileAdvise.PreLoad (currentStream);				
+					// Give the OS a hint that we will be reading this
+					// file soon.
+					FileAdvise.PreLoad (currentStream);				
+				}
 			}
 
 			try {
@@ -476,21 +605,12 @@ namespace Beagle.Daemon {
 				else if (HasError)
 					return false;
 				
-				// Close and reset our TextReader
-				if (currentReader != null) {
-					currentReader.Close ();
-					currentReader = null;
-				}
-				
-				// Seek back to the beginning of our stream
-				currentStream.Seek (0, SeekOrigin.Begin);
-				
 				DoPullSetup ();
 				
 				if (HasError)
 					return false;				
 			} catch (Exception e) {
-				Logger.Log.Warn ("Unable to filter {0}: {1}", info.FullName, e.Message);
+				Log.Warn (e, "Unable to filter {0}:", info.FullName);
 				Cleanup (); // clean up temporary files on an exception
 				return false;
 			}
@@ -533,13 +653,16 @@ namespace Beagle.Daemon {
 
 		private bool Pull ()
 		{
-			if (IsFinished) {
+			if (IsFinished || HasError) {
 				Close ();
 				return false;
 			}
 
 			DoPull ();
 
+			if (HasError)
+				return false;
+
 			return true;
 		}
 
@@ -547,21 +670,20 @@ namespace Beagle.Daemon {
 		{
 			Cleanup ();
 
-			if (currentStream == null)
-				return;
-
 			DoClose ();
 
-			// When crawling, give the OS a hint that we don't
-			// need to keep this file around in the page cache.
-			if (CrawlMode)
-				FileAdvise.FlushCache (currentStream);
-
 			if (currentReader != null)
 				currentReader.Close ();
 
-			currentStream.Close ();
-			currentStream = null;
+			if (currentStream != null) {
+				// When crawling, give the OS a hint that we don't
+				// need to keep this file around in the page cache.
+				if (CrawlMode)
+					FileAdvise.FlushCache (currentStream);
+
+				currentStream.Close ();
+				currentStream = null;
+			}
 
 			if (snippetWriter != null)
 				snippetWriter.Close ();
@@ -579,9 +701,17 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		private bool PullFromArray (ArrayList array, StringBuilder sb)
+		private bool PullFromArray (ArrayList array, StringBuilder sb, bool is_hot)
 		{
-			while (array.Count == 0 && Pull ()) { }
+			if (! is_hot) {
+				// HotText is read after Text by DoPull()*.DoClose()
+				// So, do not Pull() again for HotText - there ain't anything to pull
+				while (array.Count == 0 && Pull ()) { }
+			}
+
+			// FIXME: Do we want to try to extract as much data as
+			// possible from the filter if we get an error, or
+			// should we just give up afterward entirely?
 
 			if (array.Count > 0) {
 				foreach (string str in array)
@@ -593,14 +723,13 @@ namespace Beagle.Daemon {
 			return false;
 		}
 
-		private bool PullTextCarefully (ArrayList array, StringBuilder sb)
+		private bool PullTextCarefully (ArrayList array, StringBuilder sb, bool is_hot)
 		{
 			bool pulled = false;
 			try {
-				pulled = PullFromArray (array, sb);
+				pulled = PullFromArray (array, sb, is_hot);
 			} catch (Exception ex) {
-				Logger.Log.Debug ("Caught exception while pulling text in filter '{0}'", Name);
-				Logger.Log.Debug (ex);
+				Logger.Log.Debug (ex, "Caught exception while pulling text in filter '{0}'", Name);
 			}
 
 			return pulled;
@@ -608,18 +737,17 @@ namespace Beagle.Daemon {
 
 		private bool PullText (StringBuilder sb)
 		{
-			return PullTextCarefully (textPool, sb);
+			return PullTextCarefully (textPool, sb, false);
 		}
 
 		private bool PullHotText (StringBuilder sb)
 		{
-			return PullTextCarefully (hotPool, sb);
+			return PullTextCarefully (hotPool, sb, true);
 		}
 
 		public TextReader GetTextReader ()
 		{
 			PullingReader pr = new PullingReader (new PullingReader.Pull (PullText));
-			pr.Identifier = Identifier;
 			return pr;
 		}
 
@@ -645,6 +773,13 @@ namespace Beagle.Daemon {
 			set { uri = value; }
 		}
 
+		private Uri display_uri = null;
+
+		public Uri DisplayUri {
+			get { return display_uri; }
+			set { display_uri = value; }
+		}
+
 		//////////////////////////////
 
 		private ArrayList child_indexables = new ArrayList ();
@@ -663,4 +798,9 @@ namespace Beagle.Daemon {
 			get { return this.child_indexables; }
 		}
 	}
+
+	[AttributeUsage (AttributeTargets.Assembly)]
+	public class FilterTypesAttribute : TypeCacheAttribute {
+		public FilterTypesAttribute (params Type[] filter_types) : base (filter_types) { }
+	}
 }
