@@ -1,6 +1,8 @@
 using System;
 using Mono.Unix;
 
+using Beagle.Util;
+
 namespace Search.Tiles {
 
 	public class WebHistoryActivator : TileActivator {
@@ -40,7 +42,8 @@ namespace Search.Tiles {
 
 		public override void Open ()
 		{
-			base.OpenFromMime (Hit);
+			
+			base.OpenFromUri (UriFu.UriToEscapedString(Hit.Uri));
 		}
 
 		protected override DetailsPane GetDetails ()
@@ -48,7 +51,7 @@ namespace Search.Tiles {
 			DetailsPane details = new DetailsPane ();
 
 			details.AddLabelPair (Catalog.GetString ("Title:"), Title);
-			details.AddLabelPair (Catalog.GetString ("URL:"), Hit.UriAsString);
+			details.AddLabelPair (Catalog.GetString ("URL:"), Hit.Uri.ToString ());
 			details.AddLabelPair (Catalog.GetString ("Accessed:"), Utils.NiceLongDate (Timestamp));
 			details.AddSnippet ();
 
